@@ -15,20 +15,28 @@ ga <- function(type = c("binary", "real-valued", "permutation"),
                pcrossover = 0.8, 
                pmutation = 0.1, 
                elitism = base::max(1, round(popSize*0.05)), 
+               updatePop = FALSE,
+               postFitness = NULL,
                maxiter = 100,
                run = maxiter,
-               maxfitness = Inf,
+               maxFitness = Inf,
                names = NULL,
-               suggestions = NULL, 
+               suggestions = NULL,
+               optim = FALSE,
+               optimArgs = list(method = "L-BFGS-B", 
+                                poptim = 0.05,
+                                pressel = 0.5,
+                                control = list(fnscale = -1, maxit = 100)),
                keepBest = FALSE,
                parallel = FALSE,
-               monitor = gaMonitor,
+               monitor = if(interactive()) gaMonitor else FALSE,
                seed = NULL) 
 {
 
   call <- match.call()
   
-  type <- match.arg(type)
+  type <- match.arg(type, choices = eval(formals(ga)$type))
+  
   if(!is.function(population)) population <- get(population)
   if(!is.function(selection))  selection  <- get(selection)
   if(!is.function(crossover))  crossover  <- get(crossover)
@@ -55,6 +63,7 @@ ga <- function(type = c("binary", "real-valued", "permutation"),
   if(missing(min) & missing(max) & missing(nBits))
     { stop("A min and max range of values (for 'real-valued' or 'permutation' GA) or nBits (for 'binary' GA) must be provided!") }
   
+  # check GA search type 
   switch(type, 
          "binary"      = { nBits <- as.vector(nBits)[1]
                            min <- max <- NA
@@ -74,6 +83,7 @@ ga <- function(type = c("binary", "real-valued", "permutation"),
                          }
         )
 
+  # check suggestions
   if(is.null(suggestions))
     { suggestions <- matrix(nrow = 0, ncol = nvars) }
   else
@@ -86,13 +96,48 @@ ga <- function(type = c("binary", "real-valued", "permutation"),
         stop("Provided suggestions (ncol) matrix do not match number of variables of the problem!")
     }
 
+  # check monitor arg
+  if(is.logical(monitor))
+    { if(monitor) monitor <- gaMonitor }
+  if(is.null(monitor)) monitor <- FALSE
+  
+  # if optim merge provided and default args for optim()
+  if(optim)
+    { # merge default and provided parameters
+      optimArgs.default <- eval(formals(ga)$optimArgs)
+      optimArgs.default$control[names(optimArgs$control)] <- optimArgs$control
+      optimArgs$control <- NULL
+      optimArgs.default[names(optimArgs)] <- optimArgs
+      optimArgs <- optimArgs.default; rm(optimArgs.default)
+      if(any(optimArgs$method == c("L-BFGS-B", "Brent")))
+        { optimArgs$lower <- min
+          optimArgs$upper <- max }
+      else
+        { optimArgs$lower <- -Inf
+          optimArgs$upper <- Inf }
+      optimArgs$poptim <- min(max(0, optimArgs$poptim), 1)
+      optimArgs$pressel <- min(max(0, optimArgs$pressel), 1)
+      optimArgs$control$maxit <- as.integer(optimArgs$control$maxit)
+      # ensure that optim maximise the fitness
+      if(is.null(optimArgs$control$fnscale))
+        optimArgs$control$fnscale <- -1
+      if(optimArgs$control$fnscale > 0)
+        optimArgs$control$fnscale <- -1*optimArgs$control$fnscale
+  }
+
   # Start parallel computing (if needed)
   parallel <- if(is.logical(parallel)) 
                 { if(parallel) startParallel(parallel) else FALSE }
-              else { startParallel(parallel) }
+              else 
+                { startParallel(parallel) }
   on.exit(if(parallel)
           parallel::stopCluster(attr(parallel, "cluster")) )
-  
+  # define operator to use depending on parallel being TRUE or FALSE
+  `%DO%` <- if(parallel) `%dorng%` else `%do%`
+  # set seed for reproducibility  
+  if(!is.null(seed)) set.seed(seed)
+  i. <- NULL # dummy to trick R CMD check 
+
   fitnessSummary <- matrix(as.double(NA), nrow = maxiter, ncol = 6)
   colnames(fitnessSummary) <- names(gaSummary(rnorm(10)))
   bestSol <- if(keepBest) vector(mode = "list", length = maxiter)
@@ -119,8 +164,6 @@ ga <- function(type = c("binary", "real-valued", "permutation"),
                 summary = fitnessSummary,
                 bestSol = bestSol)
 
-  if(!is.null(seed)) set.seed(seed)
-
   # generate beginning population
   Pop <- matrix(as.double(NA), nrow = popSize, ncol = nvars)
   ng <- min(nrow(suggestions), popSize)
@@ -134,46 +177,90 @@ ga <- function(type = c("binary", "real-valued", "permutation"),
   # start iterations
   for(iter in seq_len(maxiter))
      {
+      object@iter <- iter
+
       # evalute fitness function (when needed) 
       if(!parallel)
         { for(i in seq_len(popSize))
              if(is.na(Fitness[i]))
-               { Fitness[i] <- fitness(Pop[i,], ...) } 
+               { fit <- fitness(Pop[i,], ...) 
+                 if(updatePop)
+                   Pop[i,] <- attributes(fit)[[1]]
+                 Fitness[i] <- fit
+               }
       }
       else
-        { Fitness <- foreach(i = seq_len(popSize), .combine = "c") %dopar%
-                     { if(is.na(Fitness[i])) fitness(Pop[i,], ...) 
-                       else                  Fitness[i] }
+        { Fitness <- foreach(i. = seq_len(popSize), .combine = "c") %DO%
+                     { if(is.na(Fitness[i.])) fitness(Pop[i.,], ...) 
+                       else                   Fitness[i.] }
         }
-
-      fitnessSummary[iter,] <- gaSummary(Fitness)
       
       # update object
-      object@iter <- iter
       object@population <- Pop
       object@fitness <- Fitness
-      object@summary <- fitnessSummary
       
-      if(keepBest) 
-        # object@bestSol[[iter]] <- unique(Pop[Fitness == bestEval[iter],,drop=FALSE])
-        object@bestSol[[iter]] <- unique(Pop[Fitness == fitnessSummary[iter,1],,drop=FALSE])
-
+      # update iterations summary
+      fitnessSummary[iter,] <- gaSummary(object@fitness)
+      object@summary <- fitnessSummary
+  
       if(is.function(monitor)) 
         { monitor(object) }
+      
+      # Local search optimisation
+      if(optim & (type == "real-valued"))
+        if(optimArgs$poptim > runif(1))
+        { # perform local search from random selected solution
+          # with prob proportional to fitness
+          i <- sample(1:popSize, size = 1, 
+                      prob = optimProbsel(Fitness, q = optimArgs$pressel))
+          # run local search
+          opt <- try(suppressWarnings(
+                     optim(fn = fitness, ...,
+                           par = Pop[i,],
+                           method = optimArgs$method,
+                           lower = optimArgs$lower,
+                           upper = optimArgs$upper,
+                           control = optimArgs$control)),
+                     silent = TRUE)
+          if(is.function(monitor))
+            { if(!inherits(opt, "try-error"))
+                cat(" | Local search =", 
+                    format(opt$value, digits = getOption("digits")))
+              else cat(" |", opt[1]) }
+          if(!inherits(opt, "try-error"))
+            { Pop[i,] <- opt$par
+              Fitness[i] <- opt$value }
+          # update object
+          object@population <- Pop
+          object@fitness <- Fitness
+          # update iterations summary
+          fitnessSummary[iter,] <- gaSummary(object@fitness)
+          object@summary <- fitnessSummary
+      }
+      
+      if(keepBest) 
+        { object@bestSol[[iter]] <- unique(Pop[Fitness == max(Fitness, na.rm = TRUE),,drop=FALSE]) }
+      
+      # apply a user's defined function to update the GA object
+      if(is.function(postFitness))
+        { object <- postFitness(object, ...) 
+          Fitness <- object@fitness
+          Pop <- object@population
+      }
 
       # check stopping criteria
       if(iter > 1)
-        { if(fitnessSummary[iter,1] > fitnessSummary[iter-1,1]+gaControl("eps")) 
-               object@run <- 1 
-          else 
-               object@run <- object@run + 1 
-        }
+        object@run <- garun(fitnessSummary[seq(iter),1])
+        # TODO: remove 
+        # { if(fitnessSummary[iter,1] > fitnessSummary[iter-1,1]+gaControl("eps")) 
+        #        object@run <- 1 
+        #   else 
+        #        object@run <- object@run + 1 
+        # }
       if(object@run >= run) break  
-      if(max(Fitness, na.rm = TRUE) >= maxfitness) break
+      if(max(Fitness, na.rm = TRUE) >= maxFitness) break
       if(object@iter == maxiter) break  
 
-      # PopNew <- matrix(as.double(NA), nrow = popSize, ncol = nvars)
-      # fitnessNew <- rep(NA, popSize)
       ord <- order(Fitness, decreasing = TRUE)
       PopSorted <- Pop[ord,,drop=FALSE]
       FitnessSorted <- Fitness[ord]
@@ -194,17 +281,16 @@ ga <- function(type = c("binary", "real-valued", "permutation"),
     
       # crossover
       if(is.function(crossover) & pcrossover > 0)
-        { 
-          nmating <- floor(popSize/2)
+        { nmating <- floor(popSize/2)
           mating <- matrix(sample(1:(2*nmating), size = (2*nmating)), ncol = 2)
           for(i in seq_len(nmating))
-            { if(pcrossover > runif(1))
-                { parents <- mating[i,]
-                  Crossover <- crossover(object, parents)
-                  Pop[parents,] <- Crossover$children
-                  Fitness[parents] <- Crossover$fitness
-                }
-            }             
+             { if(pcrossover > runif(1))
+                 { parents <- mating[i,]
+                   Crossover <- crossover(object, parents)
+                   Pop[parents,] <- Crossover$children
+                   Fitness[parents] <- Crossover$fitness
+                 }
+             }             
           object@population <- Pop
           object@fitness <- Fitness
         }
@@ -224,7 +310,7 @@ ga <- function(type = c("binary", "real-valued", "permutation"),
         }
 
       # elitism
-      if(elitism > 0) 
+      if(elitism > 0) # (elitism > 0 & iter > 1) 
         { ord <- order(object@fitness, na.last = TRUE)
           u <- which(!duplicated(PopSorted, margin = 1))
           Pop[ord[1:elitism],] <- PopSorted[u[1:elitism],]
@@ -232,10 +318,38 @@ ga <- function(type = c("binary", "real-valued", "permutation"),
           object@population <- Pop
           object@fitness <- Fitness
         } 
-  
+
   }
-      
-  # in case of premature convergence remove NA from summary fitness evalutations
+  
+  # if optim is required perform a local search from the best 
+  # solution at the end of GA iterations
+  if(optim & (type == "real-valued"))
+    { optimArgs$control$maxit <- rev(optimArgs$control$maxit)[1]
+      i <- which.max(object@fitness)
+      # if not provided suggest approx parscale
+      # if(is.null(optimArgs$control$parscale))
+      #   optimArgs$control$parscale <- 10^round(log10(abs(object@population[i,])+1))
+      # run local search
+      opt <- try(suppressWarnings(
+                 optim(fn = fitness, ...,
+                       par = object@population[i,],
+                       method = optimArgs$method,
+                       lower = optimArgs$lower,
+                       upper = optimArgs$upper,
+                       control = optimArgs$control)),
+                 silent = TRUE)
+      if(is.function(monitor))
+        { if(!inherits(opt, "try-error"))
+            cat(" | Final local search =",
+                format(opt$value, digits = getOption("digits")) , "\n")
+          else cat(" |", opt[1]) }
+      if(!inherits(opt, "try-error"))
+        { object@population[i,] <- opt$par
+          object@fitness[i] <- opt$value }
+  }
+   
+  # in case of premature convergence remove NAs from summary 
+  # fitness evalutations
   object@summary <- na.exclude(object@summary)
   attr(object@summary, "na.action") <- NULL
 
@@ -321,7 +435,7 @@ summary.ga <- function(object, ...)
               suggestions = suggestions,
               iter = object@iter,
               fitness = object@fitnessValue,
-              solution = object@solution)  
+              solution = object@solution)
   class(out) <- "summary.ga"
   return(out)
 }
@@ -348,12 +462,12 @@ print.summary.ga <- function(x, digits = getOption("digits"), ...)
   cat(paste("Mutation probability  = ", format(x$pmutation, digits = digits), "\n"))
   
   if(x$type == "real-valued")
-    { cat(paste("Search domain \n"))
+    { cat(paste("Search domain = \n"))
       print(x$domain, digits = digits)
     }
 
   if(!is.null(x$suggestions))
-    { cat(paste("Suggestions", "\n"))
+    { cat(paste("Suggestions =", "\n"))
       do.call(".printShortMatrix", 
               c(list(x$suggestions, digits = digits), 
                 dotargs[c("head", "tail", "chead", "ctail")]))
@@ -364,9 +478,9 @@ print.summary.ga <- function(x, digits = getOption("digits"), ...)
   cat(paste("Iterations             =", format(x$iter, digits = digits), "\n"))
   cat(paste("Fitness function value =", format(x$fitness, digits = digits), "\n"))
   if(nrow(x$solution) > 1) 
-    { cat(paste("Solutions              = \n")) }
+    { cat(paste("Solutions = \n")) }
   else
-    { cat(paste("Solution               = \n")) }
+    { cat(paste("Solution = \n")) }
   do.call(".printShortMatrix", 
           c(list(x$solution, digits = digits), 
             dotargs[c("head", "tail", "chead", "ctail")]))
@@ -378,7 +492,7 @@ print.summary.ga <- function(x, digits = getOption("digits"), ...)
 
 plot.ga <- function(x, y, ylim, cex.points = 0.7,
                     col = c("green3", "dodgerblue3", adjustcolor("green3", alpha.f = 0.1)),
-                    pch = c(16, 1), lty = c(1,2),
+                    pch = c(16, 1), lty = c(1,2), legend = TRUE,
                     grid = graphics:::grid, ...)
 {
   object <- x  # Argh.  Really want to use 'object' anyway
@@ -394,7 +508,7 @@ plot.ga <- function(x, y, ylim, cex.points = 0.7,
   plot(iters, summary[,1], type = "n", ylim = ylim, 
        xlab = "Generation", ylab = "Fitness value", ...)
   if(is.final & is.function(grid)) 
-    { grid() }
+    { grid(equilogs=FALSE) }
   points(iters, summary[,1], type = ifelse(is.final, "o", "p"),
          pch = pch[1], lty = lty[1], col = col[1], cex = cex.points)
   points(iters, summary[,2], type = ifelse(is.final, "o", "p"),
@@ -402,20 +516,21 @@ plot.ga <- function(x, y, ylim, cex.points = 0.7,
   if(is.final)
     { polygon(c(iters, rev(iters)), 
               c(summary[,4], rev(summary[,1])), 
-              border = FALSE, col = col[3])
-      legend("bottomright", legend = c("Best", "Mean"), 
-             col = col, pch = pch, lty = lty, pt.cex = cex.points, 
-             inset = 0.01) }
+              border = FALSE, col = col[3]) }
   else
     { title(paste("Iteration", object@iter), font.main = 1) }
-    
-  out <- cbind(iter = iters, summary)
+  if(is.final & legend)
+    { legend("bottomright", legend = c("Best", "Mean", "Median"), 
+             col = col, pch = c(pch,NA), lty = c(lty,1), 
+             lwd = c(1,1,10), pt.cex = c(rep(cex.points,2), 2), 
+             inset = 0.02) }
+  
+  out <- data.frame(iter = iters, summary)
   invisible(out)
 }
 
 setMethod("plot", "ga", plot.ga)
           
-# questa non funziona quando installa il pacchetto con NAMESPACE
 setGeneric(name = "parNames", 
            def = function(object, ...) { standardGeneric("parNames") }
           )
@@ -429,22 +544,29 @@ function(object, ...)
     { names <- paste("x", 1:nvars, sep = "") }
   return(names)
 })
-# per ora uso questo ma si dovrebbe ripristinare il metodo sopra:
-# gaParNames <- function(object, ...)
-# { 
-#   names <- object@names
-#   nvars <- ncol(object@population)
-#   if(length(names) == 0)
-#     { names <- paste("x", 1:nvars, sep = "") }
-#   return(names)
-# }
 
+# old function for monitoring
+gaMonitor2 <- function(object, digits = getOption("digits"), ...)
+{ 
+ fitness <- na.exclude(object@fitness)
+ cat(paste("\nIter =", object@iter, 
+           " | Mean =", format(mean(fitness), digits = digits), 
+           " | Best =", format(max(fitness), digits = digits)))
+ flush.console()
+}
+
+# new function (default) for monitoring
 gaMonitor <- function(object, digits = getOption("digits"), ...)
 { 
   fitness <- na.exclude(object@fitness)
-  cat(paste("Iter =", object@iter, 
-            " | Mean =", format(mean(fitness), digits = digits), 
-            " | Best =", format(max(fitness), digits = digits), "\n"))
+  sumryStat <- c(mean(fitness), max(fitness))
+  sumryStat <- format(sumryStat, digits = digits)
+  replicate(2, clearConsoleLine())
+  cat(paste("\rGA | iter =", object@iter,
+            # formatC(object@iter, width = nchar(object@maxiter)), 
+            "\n"))
+  cat(paste("Mean =", sumryStat[1], "| Best =", sumryStat[2]))
+  flush.console()
 }
 
 gaSummary <- function(x, ...)
